#include <vector>
#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <point3d.h>

namespace py = pybind11; // This is purely for convenience

PYBIND11_PLUGIN(points){
    py::module m("points", "A collection of functions and objects to deal with 3D points.");

    py::class_<Point3D>(m, "Point3D")
        .def(py::init<double, double, double>())
        .def(py::init<std::vector<double>>())
        .def("translate", (void (Point3D::*)(double, double, double)) &Point3D::translate,
            R"doc(Move this point by (dx, dy, dz).
        
        Parameters
        ----------
        dx : float
            shift along x-axis
        dy : float
            shift along y-axis
        dz : float
            shift along z-axis
        )doc")
        .def("coordinates", &Point3D::coordinates,
             R"doc(Get the coordinates of this point.
        
        Returns
        -------
        r : list
            coordinates of this point.
        )doc")
        .def("rotate", &Point3D::rotate, R"doc(Rotates this point about x, y, and z
        
        The rotation is performed as if this point was first rotated by alpha around the x-axis, 
        then rotated by beta around the y-axis, and finally rotated by gamma around the z-axis.
        
        Parameters
        ----------
        alpha: float
            rotation around x-axis in rad
        beta: float 
            rotation arounx y-axis in rad
        gamma: float 
            rotation around z-axis in rad
        )doc")
        ;

    return m.ptr();
}
